@@ -3,7 +3,7 @@
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "2.4.1"
+__version__ = "2.4.2"
 __all__ = ['Model', 'Wizard']
 import sys
 try:
@@ -100,6 +100,12 @@ class CharDescriptor(FieldDescriptor):
         assert isinstance(value, basestring) or value in (None, False)
         super(CharDescriptor, self).__set__(instance, value or '')
 
+    def __get__(self, instance, owner):
+        value = super(CharDescriptor, self).__get__(instance, owner)
+        if value is False:
+            value = None
+        return value
+
 
 class BinaryDescriptor(FieldDescriptor):
     default = None
