@@ -8,24 +8,65 @@ __all__ = ['set_trytond', 'set_xmlrpc', 'get_config']
 import xmlrpclib
 import threading
 from decimal import Decimal
-from types import NoneType
 import datetime
 import time
 
+
 def dump_decimal(self, value, write):
-    write("<value><double>")
-    write(str(value))
-    write("</double></value>\n")
+    value = {'__class__': 'Decimal',
+        'decimal': str(value),
+        }
+    self.dump_struct(value, write)
+
+
+def dump_buffer(self, value, write):
+    self.write = write
+    value = xmlrpclib.Binary(value)
+    value.encode(self)
+    del self.write
+
+
+def dump_date(self, value, write):
+    value = {'__class__': 'date',
+        'year': value.year,
+        'month': value.month,
+        'day': value.day,
+        }
+    self.dump_struct(value, write)
+
+
+def dump_time(self, value, write):
+    value = {'__class__': 'time',
+        'hour': value.hour,
+        'minute': value.minute,
+        'second': value.second,
+        }
+    self.dump_struct(value, write)
 
 xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
-xmlrpclib.Marshaller.dispatch[datetime.date] = \
-        lambda self, value, write: self.dump_datetime(
-                datetime.datetime.combine(value, datetime.time()), write)
+xmlrpclib.Marshaller.dispatch[datetime.date] = dump_date
+xmlrpclib.Marshaller.dispatch[datetime.time] = dump_time
+xmlrpclib.Marshaller.dispatch[buffer] = dump_buffer
 
-def _end_double(self, data):
-    self.append(Decimal(data))
+
+def end_struct(self, data):
+    mark = self._marks.pop()
+    # map structs to Python dictionaries
+    dct = {}
+    items = self._stack[mark:]
+    for i in range(0, len(items), 2):
+        dct[xmlrpclib._stringify(items[i])] = items[i + 1]
+    if '__class__' in dct:
+        if dct['__class__'] == 'date':
+            dct = datetime.date(dct['year'], dct['month'], dct['day'])
+        elif dct['__class__'] == 'time':
+            dct = datetime.time(dct['hour'], dct['minute'], dct['second'])
+        elif dct['__class__'] == 'Decimal':
+            dct = Decimal(dct['decimal'])
+    self._stack[mark:] = [dct]
     self._value = 0
-xmlrpclib.Unmarshaller.dispatch["double"] = _end_double
+
+xmlrpclib.Unmarshaller.dispatch['struct'] = end_struct
 
 _CONFIG = threading.local()
 _CONFIG.current = None
@@ -61,10 +102,15 @@ class _TrytondMethod(object):
         from trytond.cache import Cache
         from trytond.transaction import Transaction
 
-        assert self._name in self._object._rpc
+        if self._name in self._object._rpc:
+            readonly = not self._object._rpc[self._name]
+        elif self._name in getattr(self._object, '_buttons', {}):
+            readonly = False
+        else:
+            raise TypeError('%s is not callable' % self._name)
 
         with Transaction().start(self._config.database_name,
-                self._config.user) as transaction:
+                self._config.user, readonly=readonly) as transaction:
             Cache.clean(self._config.database_name)
             args = list(args)
             context = args.pop()
@@ -73,11 +119,12 @@ class _TrytondMethod(object):
                 del context['_timestamp']
             transaction.context = context
             res = getattr(self._object, self._name)(*args)
-            if self._object._rpc[self._name]:
+            if not readonly:
                 transaction.cursor.commit()
         Cache.resets(self._config.database_name)
         return res
 
+
 class TrytondProxy(object):
     'Proxy for function call for trytond'
 
@@ -147,8 +194,9 @@ class TrytondConfig(Config):
     __init__.__doc__ = object.__init__.__doc__
 
     def __repr__(self):
-        return "proteus.config.TrytondConfig('%s', '%s', '%s', config_file=%s)"\
-                % (self.database_name, self._user, self.database_type,
+        return "proteus.config.TrytondConfig"\
+            "('%s', '%s', '%s', config_file=%s)"\
+            % (self.database_name, self._user, self.database_type,
                 self.config_file)
     __repr__.__doc__ = object.__repr__.__doc__
 
@@ -171,7 +219,11 @@ class TrytondConfig(Config):
     def get_proxy_methods(self, name, type='model'):
         'Return list of methods'
         proxy = self.get_proxy(name, type=type)
-        return [x for x in proxy._object._rpc]
+        methods = [x for x in proxy._object._rpc]
+        if hasattr(proxy._object, '_buttons'):
+            methods += [x for x in proxy._object._buttons]
+        return methods
+
 
 def set_trytond(database_name=None, user='admin', database_type=None,
         language='en_US', password='', config_file=None):
@@ -194,6 +246,7 @@ class XmlrpcProxy(object):
         'Return attribute value'
         return getattr(self._object, name)
 
+
 class XmlrpcConfig(Config):
     'Configuration for XML-RPC'
 
@@ -229,10 +282,12 @@ class XmlrpcConfig(Config):
                 if x.startswith(object_)
                 and '.' not in x[len(object_) + 1:]]
 
+
 def set_xmlrpc(url):
     'Set XML-RPC as backend'
     _CONFIG.current = XmlrpcConfig(url)
     return _CONFIG.current
 
+
 def get_config():
     return _CONFIG.current
